/*
	Hyperion - A network anomaly discovery system
	Copyright (C) 2011 Nicholas Tomlinson
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "config.h"
#include "error.hpp"

#ifndef CONFIGURATION_HPP
#error include configuration.hpp, not configurationast.hpp
#endif
#ifndef CONFIGURATIONAST_HPP
#define CONFIGURATIONAST_HPP

class ConfigurationAstNode
{
	public:
		ConfigurationAstNode(){}
		
	private:
		ConfigurationAstNode(ConfigurationAstNode &);
		ConfigurationAstNode &operator=(const ConfigurationAstNode &);
};

class ConfigurationAstNodeExpression : public ConfigurationAstNode
{
	public:
		virtual ~ConfigurationAstNodeExpression();
		virtual DynamicValue GetValue(DynamicValue &root, std::string modulename) = 0;
};

class ConfigurationAstNodeAssignment : public ConfigurationAstNode
{
	public:
		ConfigurationAstNodeAssignment(std::string name, ConfigurationAstNodeExpression *expression);
		~ConfigurationAstNodeAssignment();
		void Execute(DynamicValue &root, std::string modulename);
		
	private:
		std::string name;
		ConfigurationAstNodeExpression *expression;
};

class ConfigurationAstNodeModule : public ConfigurationAstNode
{
	public:
		ConfigurationAstNodeModule(std::string name, std::vector<ConfigurationAstNodeAssignment *> assignments);
		~ConfigurationAstNodeModule();
		void Execute(DynamicValue &root);
		
	private:
		std::vector<ConfigurationAstNodeAssignment *> assignments;
		std::string name;
};

class ConfigurationAstNodeRoot : public ConfigurationAstNode
{
	public:
		ConfigurationAstNodeRoot(std::vector<ConfigurationAstNodeModule *> modules);
		~ConfigurationAstNodeRoot();
		void Execute(DynamicValue &root);
		
	private:
		std::vector<ConfigurationAstNodeModule *> modules;
};

class ConfigurationAstNodeOperator : public ConfigurationAstNodeExpression
{
	public:
		ConfigurationAstNodeOperator(char op, ConfigurationAstNodeExpression *a, ConfigurationAstNodeExpression *b);
		virtual ~ConfigurationAstNodeOperator();
		virtual DynamicValue GetValue(DynamicValue &root, std::string modulename);
		
	private:
		char op;
		ConfigurationAstNodeExpression *a;
		ConfigurationAstNodeExpression *b;
};

class ConfigurationAstNodeValue : public ConfigurationAstNodeExpression
{
	public:
		ConfigurationAstNodeValue(DynamicValue value);
		virtual DynamicValue GetValue(DynamicValue &root, std::string modulename);
		
	private:
		DynamicValue value;
};

class ConfigurationAstNodeValueList : public ConfigurationAstNodeExpression
{
	public:
		ConfigurationAstNodeValueList(std::vector<ConfigurationAstNodeExpression *> expressions);
		virtual ~ConfigurationAstNodeValueList();
		virtual DynamicValue GetValue(DynamicValue &root, std::string modulename);
		
	private:
		std::vector<ConfigurationAstNodeExpression *> expressions;
};

class ConfigurationAstNodeVariable : public ConfigurationAstNodeExpression
{
	public:
		ConfigurationAstNodeVariable(std::string name);
		
		virtual DynamicValue GetValue(DynamicValue &root, std::string modulename);
		
	private:
		std::string name;
};




struct ParserData
{
	const char *filename;
	void *scanner;
	ConfigurationAstNodeRoot *root;
	bool error;
};

#define YYSTYPE_CA_LEN 1024
struct YYSTYPE {
	char chararray[YYSTYPE_CA_LEN];
	const char *text;
	std::string str;
	
	ConfigurationAstNodeModule *module;
	ConfigurationAstNodeAssignment *assignment;
	ConfigurationAstNodeExpression *expression;
	
	std::vector<ConfigurationAstNodeModule *> modules;
	std::vector<ConfigurationAstNodeAssignment *> assignments;
	std::vector<ConfigurationAstNodeExpression *> expressions;
};
typedef struct YYSTYPE YYSTYPE;
#define YYSTYPE_IS_DECLARED


#endif