/*
	Hyperion - A network anomaly discovery system
	Copyright (C) 2011 Nicholas Tomlinson
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "config.h"
#include "error.hpp"

#ifndef DATABASE_HPP
#define DATABASE_HPP

#include "configuration.hpp"
#include "dynamic.hpp"

struct sqlite3;
class EventManager;
class QueryObject;
class Query
{
	friend class Database;
	public:
		Query();
		~Query();
		Query(const Query &rhs);
		Query &operator=(const Query &rhs);
		DynamicValue operator()();
		DynamicValue operator()(DynamicValue v);
		
	private:
		Query(QueryObject *po);
		QueryObject *o;
};

class Database
{
	public:
		Database(Configuration &configuration);
		~Database();
		void Init(EventManager *manager);
		Query operator()(std::string sql);
		
	private:
		Database(const Database &);
		Database &operator=(const Database &);
		
		Configuration &configuration;
		EventManager *manager;
		sqlite3 *db;
};

#endif
