/*
	Hyperion - A network anomaly discovery system
	Copyright (C) 2011-2012 Nicholas Tomlinson
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "config.h"
#include "error.hpp"

#ifndef DYNAMIC_HPP
#define DYNAMIC_HPP

#include <stdint.h>

#include <map>
#include <string>
#include <vector>

#include "buffer.hpp"
#include "format.hpp"
#include "time.hpp"

//TODO: This class will want a speedup (maybe even making it (and its subclasses) inline)
//TODO: Improve the string formatting (and make ToString() optional, and so on)
class Event;
class DynamicValueDataBase;
class DynamicValue
{
	public:
		DynamicValue();
		DynamicValue(bool v);
		DynamicValue(int8_t v);
		DynamicValue(uint8_t v);
		DynamicValue(int16_t v);
		DynamicValue(uint16_t v);
		DynamicValue(int32_t v);
		DynamicValue(uint32_t v);
		DynamicValue(int64_t v);
		DynamicValue(uint64_t v);
		DynamicValue(const char *v);
		DynamicValue(void *v);
		DynamicValue(double v);
		DynamicValue(std::string v);
		DynamicValue(Time v);
		DynamicValue(Buffer v);
		DynamicValue(Event v);
		
		DynamicValue(std::map<std::string, bool> v);
		DynamicValue(std::map<std::string, int8_t> v);
		DynamicValue(std::map<std::string, uint8_t> v);
		DynamicValue(std::map<std::string, int16_t> v);
		DynamicValue(std::map<std::string, uint16_t> v);
		DynamicValue(std::map<std::string, int32_t> v);
		DynamicValue(std::map<std::string, uint32_t> v);
		DynamicValue(std::map<std::string, int64_t> v);
		DynamicValue(std::map<std::string, uint64_t> v);
		DynamicValue(std::map<std::string, const char *>v);
		DynamicValue(std::map<std::string, void *>v);
		DynamicValue(std::map<std::string, double> v);
		DynamicValue(std::map<std::string, std::string> v);
		DynamicValue(std::map<std::string, Time> v);
		DynamicValue(std::map<std::string, Buffer> v);
		DynamicValue(std::map<std::string, Event> v);
		
		DynamicValue(std::vector<DynamicValue> v);
		DynamicValue(std::map<std::string, DynamicValue> v);
		DynamicValue(const DynamicValue &v);
		DynamicValue &operator=(const DynamicValue &v);
		virtual ~DynamicValue();
		
		DynamicValue Op(char op, const DynamicValue &rhs) const;
		
		operator bool();
		operator int8_t();
		operator uint8_t();
		operator int16_t();
		operator uint16_t();
		operator int32_t();
		operator uint32_t();
		operator int64_t();
		operator uint64_t();
		operator void *();
		operator double();
		operator std::string();
		operator Time();
		operator Buffer();
		operator Event();
		DynamicValue &operator[](unsigned int i);
		DynamicValue &operator[](const char *key);
		DynamicValue &operator[](std::string key);
		bool Has(std::string key);
		std::string ToString();
		operator std::vector<DynamicValue>();
		operator std::map<std::string, DynamicValue>();
		void SetRoKey();
		void Format(DynamicValueFormat format);
		
		template <typename T> bool Type(T *p = 0)
		{
			Error::InvertCrash();
			try {
				T v = *this;
				if (p){
					*p = v;
				}
				Error::InvertCrash();
				return true;
			}
			catch (Error e){
				Error::InvertCrash();
				return false;
			}
			return false;
		}
		
	private:
		//These have been put here so I can make this more efficient later
		std::vector<DynamicValue> &GetVector();
		std::map<std::string, DynamicValue> &GetMap();
		
		DynamicValueDataBase *data;
		DynamicValueFormat format;
		bool ro;
};

#endif
