/*
	Hyperion - A network anomaly discovery system
	Copyright (C) 2011 Nicholas Tomlinson
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "config.h"

#include <cstdio>
#include <stdarg.h>
#include "error.hpp"

bool Error::crash = true;

void Error::InvertCrash()
{
	crash = !crash;
}

void PrintLocatedError(const char *filename, int l1, int c1, int l2, int c2, const char *format, ...)
{
	char str[ERROR_MSG_SIZE];
	
	va_list argp;
	va_start(argp, format);
	vsnprintf(str, ERROR_MSG_SIZE, format, argp);
	va_end(argp);
	
	fprintf(stderr, "Line %i: %s\n", l2, str);
}

Error::Error(const char *pfile, int pline, const char *format, ...)
{
	file = pfile;
	line = pline;
	
	va_list argp;
	va_start(argp, format);
	vsnprintf(str, ERROR_MSG_SIZE, format, argp);
	va_end(argp);
	
	if (crash){
		Crash();
	}
}

void Error::OutputMessage(bool pos, FILE *f)
{
	if (pos){
		fprintf(f, "Error in file %s at line %i: %s\n", file, line, str);
	}
	else {
		fprintf(f, "Error: %s\n", str);
	}
}

//BUG: If this is called twice with pos=true, the filename/line will be included twice
const char *Error::GetMessage(bool pos)
{
	char tmp[ERROR_MSG_SIZE]; //temporary buffer
	snprintf(tmp, ERROR_MSG_SIZE, "%s", str); //copy the message into the temporary
	if (pos){
		//format into str (which doesn't disappear at the end of this function)
		snprintf(str, ERROR_MSG_SIZE, "Error in file %s at line %i: %s\n", file, line, tmp);
	}
	return str;
}

//This is a hack, but it's useful to force gdb to give us a backtrace
//(at least, without having to set up a break point by hand)
void Error::Crash()
{
#ifdef DEBUG
	fprintf(stderr, "[****] Error in file %s at line %i: %s [****]\n", file, line, str);
	
	int *p = 0;
	p[0] = 0;
#endif
}