/*
	Hyperion - A network anomaly discovery system
	Copyright (C) 2011 Nicholas Tomlinson
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

/*
 * This is based on an error handling class I already have
*/

#ifndef ERROR_HPP
#define ERROR_HPP

#include "config.h"

#include <cstdio>
#include <cstring>
#include <errno.h>

#ifdef DEBUG
	#define ERROR_POS_DEFAULT true
#else
	#define ERROR_POS_DEFAULT false
#endif

#define FormatError(format, args...) Error(__FILE__, __LINE__, format, ## args)
#define errstr strerror(errno)

#ifdef DEBUG
	#define AssertError(condition, format, args...) if (!(condition)) throw Error(__FILE__, __LINE__, format, ## args)
#else
	#define AssertError(condition, format, args...) /*Do nothing*/
#endif

void PrintLocatedError(const char *filename, int l1, int c1, int l2, int c2, const char *format, ...);
#define PrintCompilerErrorYyl(filename, yyl, args...)\
	PrintLocatedError(filename, yyl.first_line, yyl.first_column, yyl.last_line, yyl.last_column, ## args);

#define ERROR_MSG_SIZE 4096
class Error
{
	public:
		Error(const char *file, int line, const char *format, ...);
		
		void OutputMessage(bool pos = ERROR_POS_DEFAULT, FILE *f = stderr);
		const char *GetMessage(bool pos = ERROR_POS_DEFAULT);
		
		static void InvertCrash();
		
	private:
		void Crash();
		
		char str[ERROR_MSG_SIZE];
		const char *file;
		int line;
		static bool crash;
};

//TODO: make something that allows us to strerr(errno()) easily

#endif