/*
	Hyperion - A network anomaly discovery system
	Copyright (C) 2011 Nicholas Tomlinson
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "config.h"
#include "error.hpp"

#include "eventloop.hpp"
#include "socket.hpp"
#include "timer.hpp"

EventLoop::EventLoop(Configuration *pconfiguration, Database *pdatabase)
{
	configuration = pconfiguration;
	database = pdatabase;
	modules = new Modules(configuration, database);
	manager = new EventManager(modules);
	database->Init(manager);
	
	socketmodule = new SocketModule(modules, manager, configuration, database);
	timermodule = new TimerModule(modules, manager, configuration, database);
	
	modules->AddModule(socketmodule);
	modules->AddModule(timermodule);
	
	modules->InitModules(manager);
}

EventLoop::~EventLoop()
{
	delete manager;
	delete modules; //the manager MUST be deleted last
		//(otherwise the destructors will have been dlclosed)
}

void EventLoop::Loop(Time testtime)
{
	Time start = Time::now();
	while ((testtime < Time::zero) || (Time::now() < start + testtime)){
		socketmodule->ProcessSockets(timermodule->ProcessTimers());
	}
}
