/*
	Hyperion - A network anomaly discovery system
	Copyright (C) 2011-2012 Nicholas Tomlinson
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "config.h"
#include "error.hpp"

#ifndef FSM_HPP
#define FSM_HPP

template <typename C, typename D> class FsmNode
{
	public:
		virtual FsmNode<C, D> *ProcessCharacter(C c, D &d) = 0;
		
		virtual FsmNode<C, D> *ProcessTransitionIn(D &d)
		{
			return this; //by default, do nothing
		}
		
	private:
};

template <typename C, typename D> class Fsm
{
	public:
		Fsm(D data, FsmNode<C, D> *first) : d(data)
		{
			node = first;
			Transition();
		}
		
		virtual ~Fsm()
		{
		}
		
		void ProcessCharacter(C c)
		{
			if (!node){ //allow null so that it is an error to process more characters in this state
				//(useful if we wish to have nodes that end a protocol, or something like that)
				throw FormatError("Attempted to process a character in the null FSM state!");
			}
			
			FsmNode<C, D> *n = node->ProcessCharacter(c, d);
			if (n != node){ //this way we can return this after having deleted this
				//without accessing deallocated memory
				//(also useful at the end of a protocol)
				
				//If ever we implement transition out, process it here
				node = n;
				Transition();
			}
		}
		
		void Transition()
		{
			while (node){
				FsmNode<C, D> *n = node->ProcessTransitionIn(d);
				if (n == node){
					break;
				}
				else {
					node = n;
				}
			}
		}
		
		D GetData()
		{
			return d;
		}
		
		//This could be useful for detecting when this is zero
		FsmNode<C, D> *GetNode()
		{
			return node;
		}
		
	private:
		FsmNode<C, D> *node;
		D d;
};

#endif
