/*
	Hyperion - A network anomaly discovery system
	Copyright (C) 2011 Nicholas Tomlinson
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "config.h"
#include "error.hpp"

#include <cstdio>
#include <cstdlib>
#include <string>
#include <ctime>

#include <grp.h>
#include <signal.h>

#include "configuration.hpp"
#include "database.hpp"
#include "error.hpp"
#include "eventloop.hpp"

//TODO: Implement signal handler for ctrl+c
int main(int argc, const char *argv[])
{
	try
	{
		/*Initialise*/
		srand(time(0));
		Configuration configuration(argc-1, argv + 1);
		Database database(configuration);
		EventLoop loop(&configuration, &database);
		
		/*Lose root privilages*/
		bool droproot = configuration["hyperion"]["droproot"];
		if ((!getuid() || !getgid()) && droproot){
			unsigned int uid = configuration["hyperion"]["uid"];
			unsigned int gid = configuration["hyperion"]["gid"];
			
			if (!uid){
				throw FormatError("Cannot drop to root user!");
			}
			
			if (!gid){
				throw FormatError("Cannot drop to root group!");
			}
			
			printf("Dropping root privilages to uid %u and gid %u\n", uid, gid);
			
			gid_t sg[1];
			if(setgroups(0, sg)){
				throw FormatError("Could not drop supplementary groups!");
			}
			
			if (setgid(gid)){
				throw FormatError("setgid(): %s", errstr);
			}
			
			if (setuid(gid)){
				throw FormatError("setuid(): %s", errstr);
			}
		}
		else if ((!getuid() || !getgid()) && !droproot){
			printf("WARNING: running as root!\n");
		}
		
		/*Loop*/
		loop.Loop();
		//loop.Loop(1); //TODO ^^^^
	}
	catch (Error e)
	{
		e.OutputMessage();
		return 1;
	}
	
	return 0;
}