/*
	Hyperion - A network anomaly discovery system
	Copyright (C) 2011 Nicholas Tomlinson
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "config.h"
#include "error.hpp"

#ifndef MODULE_HPP
#define MODULE_HPP

#include "configuration.hpp"
#include "database.hpp"
#include "event.hpp"


//TODO: change InitModule() so it adds modules to the manager rather than returning a moduleclass
//		This will allow more than module class per so
#ifdef MODULES
#define INIT_MODULE(moduleclass) \
	extern "C" Module *InitModule \
		(Modules *modules, EventManager *manager, Configuration *configuration, Database *database) \
	{ \
		return new moduleclass(modules, manager, configuration, database); \
	};
#else
#define INIT_MODULE(moduleclass) \
	Module *InitModule_##moduleclass \
		(Modules *modules, EventManager *manager, Configuration *configuration, Database *database) \
	{ \
		return new moduleclass(modules, manager, configuration, database); \
	};
#endif

class Modules;
class Module
{
	public:
		Module(const char *name, Modules *modules, EventManager *manager, Configuration *configuration, Database *database);
		virtual ~Module();
		
		const char *GetName();
		
	protected:
		Configuration &GetConfiguration();
		Modules *GetModules();
		EventManager *GetManager();
		Database &GetDatabase();
		
	private:
		Module(const Module &);
		Module &operator=(const Module &);
		
		Configuration *configuration;
		Modules *modules;
		EventManager *manager;
		Database *database;
		const char *name;
};

#endif
