/*
	Hyperion - A network anomaly discovery system
	Copyright (C) 2011 Nicholas Tomlinson
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "config.h"
#include "error.hpp"

#ifndef MODULES_HPP
#define MODULES_HPP

#include <string>
#include <map>

#include "configuration.hpp"
#include "database.hpp"
#include "event.hpp"
#include "module.hpp"

class Module;
class EventManager;

struct ModulesItem
{
	Module *module;
	std::string path;
	void *handle;
};

class Modules
{
	public:
		Modules(Configuration *configuration, Database *database);
		virtual ~Modules();
		
		void LoadModule(EventManager *manager, const char *path);
		void InitModules(EventManager *manager, const char *path = 0);
		void AddModule(Module *module);
		Module *GetModule(std::string name);
		
	private:
		Modules(const Modules &);
		Modules &operator=(const Modules &);
		
		void AddModule(Module *module, const char *path, void *handle);
		
		Configuration *configuration;
		Database *database;
		std::map<std::string, ModulesItem> modules;
};

#endif
