/*
	Hyperion - A network anomaly discovery system
	Copyright (C) 2011 Nicholas Tomlinson
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "config.h"
#include "error.hpp"

#include <cstdio>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>

#include "socket.hpp"

#include "buffer.hpp"

class TestModule : public SocketModuleBase
{
	public:
		TestModule(Modules *modules, EventManager *manager, Configuration *configuration, Database *database) :
			SocketModuleBase("Test", modules, manager, configuration, database)
		{
			//Note: these must be openned in this order in the other script too to avoid blocking
			std::string in_filename = GetConfiguration()["test"]["in"];
			std::string out_filename = GetConfiguration()["test"]["out"];
			
			in_file = open(in_filename.c_str(), O_RDONLY);
			if (in_file < 0){
				throw FormatError("open(): %s", errstr);
			}
			
			out_file = open(out_filename.c_str(), O_WRONLY);
			if (out_file < 0){
				throw FormatError("open(): %s", errstr);
			}
			
			AddSocket(in_file);
			GetManager()->Subscribe(GetCallbackNameEventSubscription<TestModule>
				("send:sniff", this, &TestModule::OnSend));
		}
		
		//add a destructor to close the sock
		
		//this might need to be changed to support multiple sockets
		//(i.e. add a parameter (e.g. an int sock))
		virtual void HandleSocketEvent()
		{
			uint16_t packet_length = 0;
			int length = read(in_file, &packet_length, 2);
			if (length < 0){
				throw FormatError("read(): %s", errstr);
			}
			else if (!length){
				throw FormatError("read() returned zero");
			}
			else if (length != 2){
				throw FormatError("read() returned non-four");
			}
			else if (!packet_length){
				throw FormatError("Packet length is zero");
			}
			
			Buffer b;
			unsigned char *buffer = b.GetBuffer(packet_length);
			
			//TODO: maybe we can change this to get a sockaddr_ll that would
			//tell us the direction of the packet?
			length = read(in_file, buffer, packet_length);
			if (length < 0){
				throw FormatError("read(): %s", errstr);
			}
			else if (!length){
				throw FormatError("read() returned zero");
			}
			else if (length != packet_length){
				throw FormatError("read() could not read entire packet");
			}
			
			b = Buffer(b, 0, length);
			
			Event e;
			e["received:ethernet"]["data"] = b;
			e["sniff"]["data"] = b;
			GetManager()->Publish(e);
		}
		
		void OnSend(Event event)
		{
			Buffer payload = event["send:sniff"]["payload"];
			if (write(out_file, payload.GetBuffer(), payload.GetLength()) < 0){
				throw FormatError("write(): %s", errstr);
			}
		}
		
	private:
		int in_file;
		int out_file;
};

INIT_MODULE(TestModule);
