/*
	Hyperion - A network anomaly discovery system
	Copyright (C) 2012 Nicholas Tomlinson
	
	This program is free software: you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation, either version 3 of the License, or
	(at your option) any later version.
	
	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.
	
	You should have received a copy of the GNU General Public License
	along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

#include "config.h"
#include "error.hpp"

#include <vector>
#include <pcrecpp.h>

#include "event.hpp"
#include "module.hpp"

class TracerouteAnomaliesModule : public Module
{
	public:
		TracerouteAnomaliesModule(Modules *modules, EventManager *manager, Configuration *configuration, Database *database) :
			Module("Traceroute Anomalies", modules, manager, configuration, database),
			badip(GetConfiguration()["tracerouteanomalies"]["blacklist"])
		{
			GetManager()->Subscribe(GetCallbackNameEventSubscription<TracerouteAnomaliesModule>
				("traceroute:new traceroute", this, &TracerouteAnomaliesModule::OnReceiveTraceroute));
		}
		
	private:
		void OnReceiveTraceroute(Event event)
		{
			double maxrtt = GetConfiguration()["tracerouteanomalies"]["maxrtt"];
			DynamicValue ev;
			ev["source"] = event["traceroute:new traceroute"]["source"];
			ev["destination"] = event["traceroute:new traceroute"]["destination"];
			ev["traceroute"] = event["traceroute:new traceroute"]["list"];
			std::vector<DynamicValue> traceroute = event["traceroute:new traceroute"]["list"];
			for (std::vector<DynamicValue>::iterator it = traceroute.begin(); it != traceroute.end(); it++){
				DynamicValue response = *it;
				DynamicValue evi = ev;
				evi["node"] = response["address"];
				evi["ttl"] = response["ttl"];
				evi["rtt"] = response["rtt"];
				
				if (badip.FullMatch(response["address"].ToString())){
					Event e;
					e["anomaly:traceroute:blacklist"] = evi;
					GetManager()->Publish(e);
				}
				
				double rtt = response["rtt"];
				//rtt /= 1000000; //get it in ms
				
				//printf("RTT: %s, %s, %f, %f\n", event["traceroute:new traceroute"]["destination"].ToString().c_str(), response["address"].ToString().c_str(), rtt, maxrtt);
				
				if (rtt > maxrtt){
					Event e;
					e["anomaly:traceroute:rtt"] = evi;
					GetManager()->Publish(e);
				}
			}
		}
		
		pcrecpp::RE badip;
};

INIT_MODULE(TracerouteAnomaliesModule);
